# @author Eric Dürr


require "lppP6/version"

Node = Struct.new(:value, :next, :prev) do
    include Comparable

    def <=>(other)
        return nil unless other.instance_of?Node
        
        value <=> other.value 
    end

    def to_s
        "[ |#{value}| ]"
    end
end


class DobLinkedList


    include Comparable, Enumerable

    attr_reader :head, :tail, :length
    
    def initialize
        @head   = nil
        @tail   = nil
        @length = 0
    end

    def to_s
        listString = ""
        current = @head
        while current

            listString = listString + current.to_s 
            current = current.next
        end

        return listString
    end


    def insertHead(value)
        node = Node.new(value, nil, nil)
        
        if @head
            node.next = @head
            @head.prev = node
            @head = node
        else
            @head = node
        end
        @length += 1
    end

    
    def insertTail(value)
        node = Node.new(value, nil, nil)
        
        unless @head
            @head = node
        else
            node.prev = @tail
            @tail.next = node
        end
        @tail = node
        @length += 1
    end


    def insertMany(valuesArray)

        valuesArray.each {| value |   
      
        node = Node.new(value, nil, nil)
        
        unless @head
            @head = node
        else
            node.prev = @tail
            @tail.next = node
        end
        @tail = node
        @length += 1
        
        }
    end

    def remove_head

        if @head.next.nil?
            @head = @tail = nil
        else
            @head = @head.next
            @head.prev = nil
        end

        @length -= 1
    end

    def remove_tail

        if @tail.prev.nil?
            @tail = @head = nil

        else

            @tail = @tail.prev
            @tail.next = nil
        end

        @length -= 1
    end

    def remove(node)
        return nil unless node    
        
        if node == @head
            if @head.next.nil?
                @head = @tail = nil
            else
                @head = @head.next
            end
        else
            pr = node.prev
            ne = node.next
            pr&.next = ne
            ne&.prev = pr
        end
        @length -= 1
    end

    def <=>(other)
        return nil unless other.instance_of? DobLinkedList
        
        value <=> other.value 
    end
    
    def find_first(&predicate)
        return nil unless block_given?    
        current = @head
        while current
            return current if predicate.call(current)
            current = current.next
        end
    end


    def each
        return nil unless block_given?    
        current = @head
        while current
            yield current
            current = current.next
        end
    end

    def find_node (index)
        current = @head
        count = 0
        while current
            return current if count==index
            count += 1
            current = current.next
        end
    end

    def [](index)
        case index
            when 0, -@length
                @head
            when length-1, -1
                @tail
            else
                find_node(index)
        end
    end


    

end